@@ -1,3 +1,4 @@
+import os
 from flask import request
 from typing import Callable
 
@@ -36,7 +37,23 @@ class DeleteRoute(RouteBase):
             handle_exception=handle_exception,
         )
 
-    def handle_request(self, id):
+    def _partition_filters(self, args):
+        partition_filters = {}
+        # Tratando campos de particionamento
+        for field in self._dto_class.partition_fields:
+            value = args.get(field)
+            if value is None:
+                raise MissingParameterException(field)
+            partition_filters[field] = value
+
+        return partition_filters
+
+    def handle_request(
+        self,
+        id: str = None,
+        query_args: dict[str, any] = None,
+        body: dict[str, any] = None,
+    ):
         """
         Tratando requisições HTTP Delete para excluir uma instância de uma entidade.
         """
@@ -44,26 +61,35 @@ class DeleteRoute(RouteBase):
         with self._injector_factory() as factory:
             try:
                 # Recuperando os parâmetros básicos
-                args = request.args
+                if os.getenv("ENV", "").lower() != "erp_sql":
+                    args = request.args
+                else:
+                    args = query_args
 
-                partition_fields = {}
-                # Tratando campos de particionamento
-                for field in self._dto_class.partition_fields:
-                    value = args.get(field)
-                    if value is None:
-                        raise MissingParameterException(field)
-                    
-                    partition_fields[field] = value
+                # Recuperando os dados do corpo da requisição
+                request_data = request.json
+
+                if request_data is not None:
+                    if not isinstance(request_data, list):
+                        request_data = [request_data]
+
+                partition_filters = self._partition_filters(args)
 
                 # Construindo os objetos
                 service = self._get_service(factory)
 
-                # Chamando o service (método get)
-                # TODO Rever parametro order_fields abaixo
-                service.delete(id, partition_fields)
+                if id is not None:
+                    # Chamando o service (método get)
+                    # TODO Rever parametro order_fields abaixo
+                    service.delete(id, partition_filters)
+                else:
+                    service.delete_list(
+                        request_data,
+                        partition_filters
+                    )
 
                 # Retornando a resposta da requuisição
-                return ('', 204, {**DEFAULT_RESP_HEADERS})
+                return ("", 204, {**DEFAULT_RESP_HEADERS})
             except MissingParameterException as e:
                 get_logger().warning(e)
                 if self._handle_exception is not None:
