@@ -1,3 +1,5 @@
+import os
+
 from flask import request
 from typing import Callable
 
@@ -36,7 +38,11 @@ class ListRoute(RouteBase):
             handle_exception=handle_exception,
         )
 
-    def handle_request(self):
+    def handle_request(
+        self,
+        query_args: dict[str, any] = None,
+        body: dict[str, any] = None,
+    ):
         """
         Tratando requisições HTTP Get (para listar entidades, e não para recuperar pelo ID).
         """
@@ -44,19 +50,39 @@ class ListRoute(RouteBase):
         with self._injector_factory() as factory:
             try:
                 # Recuperando os parâmetros básicos
-                base_url = request.base_url
-                args = request.args
+                if os.getenv("ENV", "").lower() != "erp_sql":
+                    base_url = request.base_url
+                    args = request.args
+                else:
+                    base_url = ""
+                    args = query_args
+
                 limit = int(args.get("limit", DEFAULT_PAGE_SIZE))
                 current_after = args.get("after") or args.get("offset")
 
                 # Tratando dos fields
                 fields = args.get("fields")
                 fields = self._parse_fields(fields)
-                url_args = base_url + '?' + '&'.join([f'{arg}={value}' for arg, value in args.items() if arg not in ["limit", "after", "offset", "fields"]])
+                url_args = (
+                    base_url
+                    + "?"
+                    + "&".join(
+                        [
+                            f"{arg}={value}"
+                            for arg, value in args.items()
+                            if arg not in ["limit", "after", "offset"]
+                        ]
+                    )
+                )
 
-                # Tratando dos filters
+                # Tratando dos filters e search_query
                 filters = {}
+                search_query = None
                 for arg in args:
+                    if arg.lower() == "search":
+                        search_query = args.get(arg)
+                        continue
+
                     if arg in ["limit", "after", "offset", "fields"]:
                         continue
 
@@ -78,11 +104,21 @@ class ListRoute(RouteBase):
 
                 # Chamando o service (método list)
                 # TODO Rever parametro order_fields abaixo
-                data = service.list(current_after, limit, fields, None, filters)
+                data = service.list(
+                    current_after,
+                    limit,
+                    fields,
+                    None,
+                    filters,
+                    search_query=search_query,
+                )
 
                 # Convertendo para o formato de dicionário (permitindo omitir campos do DTO)
                 dict_data = [dto.convert_to_dict(fields) for dto in data]
 
+                # Recuperando o campo referente à chave primária do DTO
+                pk_field = self._dto_class.pk_field
+
                 # Construindo o corpo da página
                 page = page_body(
                     base_url=url_args,
@@ -90,7 +126,7 @@ class ListRoute(RouteBase):
                     current_after=current_after,
                     current_before=None,
                     result=dict_data,
-                    id_field="id",  # TODO Rever esse parâmetro
+                    id_field=pk_field,
                 )
 
                 # Retornando a resposta da requuisição
