@@ -1,5 +1,9 @@
+import decimal
+import datetime
 import enum
 import uuid
+import re
+import unidecode
 
 from typing import Any, Dict, List, Tuple
 
@@ -12,6 +16,7 @@ from nsj_rest_lib.exception import (
     NotFoundException,
     AfterRecordNotFoundException,
 )
+from nsj_rest_lib.util.join_aux import JoinAux
 
 from nsj_gcf_utils.db_adapter2 import DBAdapter2
 from nsj_gcf_utils.json_util import convert_to_dumps
@@ -56,7 +61,7 @@ class DAOBase:
         """
         return self._db.in_transaction()
 
-    def _sql_fields(self, fields: List[str] = None) -> str:
+    def _sql_fields(self, fields: List[str] = None, table_alias: str = "t0") -> str:
         """
         Returns a list of fields to build select queries (in string, with comma separator)
         """
@@ -72,8 +77,8 @@ class DAOBase:
                 if not callable(getattr(entity, k, None)) and not k.startswith("_")
             ]
 
-        resp = ", t0.".join(fields)
-        return f"t0.{resp}"
+        resp = f", {table_alias}.".join(fields)
+        return f"{table_alias}.{resp}"
 
     def get(
         self,
@@ -83,6 +88,7 @@ class DAOBase:
         filters=None,
         conjunto_type: ConjuntoType = None,
         conjunto_field: str = None,
+        joins_aux: List[JoinAux] = None,
     ) -> EntityBase:
         """
         Returns an entity instance by its ID.
@@ -107,15 +113,20 @@ class DAOBase:
         # Organizando o where dos filtros
         filters_where, filter_values_map = self._make_filters_sql(filters)
 
+        # Montando a clausula dos fields vindos dos joins
+        sql_join_fields, sql_join = self._make_joins_sql(joins_aux)
+
         # Building query
         sql = f"""
         {with_conjunto}
         select
             {fields_conjunto}
             {self._sql_fields(fields)}
+            {sql_join_fields}
         from
             {entity.get_table_name()} as t0
             {join_conjuntos}
+            {sql_join}
         where
             t0.{key_field} = :id
             {filters_where}
@@ -158,98 +169,126 @@ class DAOBase:
 
         filters_where = ""
         filter_values_map = {}
-        if filters is not None:
-            filters_where = []
 
-            # Iterating fields with filters
-            for filter_field in filters:
-                field_filter_where_or = []
-                field_filter_where_and = []
+        if filters is None:
+            return (filters_where, filter_values_map)
 
-                # Iterating condictions
-                idx = -1
-                for condiction in filters[filter_field]:
-                    idx += 1
+        filters_where = []
 
-                    # Resolving condiction
-                    operator = "="
-                    if condiction.operator == FilterOperator.DIFFERENT:
-                        operator = "<>"
-                    elif condiction.operator == FilterOperator.GREATER_THAN:
-                        operator = ">"
-                    elif condiction.operator == FilterOperator.LESS_THAN:
-                        operator = "<"
-                    elif condiction.operator == FilterOperator.GREATER_OR_EQUAL_THAN:
-                        operator = ">="
-                    elif condiction.operator == FilterOperator.LESS_OR_EQUAL_THAN:
-                        operator = "<="
-                    elif condiction.operator == FilterOperator.LIKE:
-                        operator = "like"
-                    elif condiction.operator == FilterOperator.ILIKE:
-                        operator = "ilike"
-                    elif condiction.operator == FilterOperator.NOT_NULL:
-                        operator = "is not null"
+        # Iterating fields with filters
+        for filter_field in filters:
+            field_filter_where_or = []
+            field_filter_where_and = []
+            field_filter_where_in = []
+            field_filter_where_not_in = []
+            table_alias = "t0"
 
-                    # Making condiction alias
-                    if not (condiction.operator == FilterOperator.NOT_NULL):
-                        condiction_alias = (
-                            f"ft_{condiction.operator.value}_{filter_field}_{idx}"
-                        )
-                        condiction_alias_subtituir = f":{condiction_alias}"
+            # Iterating condictions
+            idx = -1
+            for condiction in filters[filter_field]:
+                idx += 1
+
+                if condiction.table_alias is not None:
+                    table_alias = condiction.table_alias
+
+                # Resolving condiction
+                operator = "="
+                if condiction.operator == FilterOperator.DIFFERENT:
+                    operator = "<>"
+                elif condiction.operator == FilterOperator.GREATER_THAN:
+                    operator = ">"
+                elif condiction.operator == FilterOperator.LESS_THAN:
+                    operator = "<"
+                elif condiction.operator == FilterOperator.GREATER_OR_EQUAL_THAN:
+                    operator = ">="
+                elif condiction.operator == FilterOperator.LESS_OR_EQUAL_THAN:
+                    operator = "<="
+                elif condiction.operator == FilterOperator.LIKE:
+                    operator = "like"
+                elif condiction.operator == FilterOperator.ILIKE:
+                    operator = "ilike"
+                elif condiction.operator == FilterOperator.NOT_NULL:
+                    operator = "is not null"
+
+                # Making condiction alias
+                if not (condiction.operator == FilterOperator.NOT_NULL):
+                    condiction_alias = (
+                        f"ft_{condiction.operator.value}_{filter_field}_{idx}"
+                    )
+                    condiction_alias_subtituir = f":{condiction_alias}"
+                else:
+                    condiction_alias = ""
+                    condiction_alias_subtituir = ""
+
+                # Making condiction buffer
+                if use_table_alias:
+                    condiction_buffer = f"{table_alias}.{filter_field} {operator} {condiction_alias_subtituir}"
+                else:
+                    condiction_buffer = (
+                        f"{filter_field} {operator} {condiction_alias_subtituir}"
+                    )
+
+                multiple_values = len(filters[filter_field]) > 1 or (
+                    isinstance(condiction.value, set) and len(condiction.value) > 1
+                )
+
+                # Storing field filter where
+                if operator == "=" and multiple_values:
+                    field_filter_where_in.append(condiction_alias_subtituir)
+                elif operator == "<>" and multiple_values:
+                    field_filter_where_not_in.append(condiction_alias_subtituir)
+                elif operator == "=" or operator == "like" or operator == "ilike":
+                    field_filter_where_or.append(condiction_buffer)
+                else:
+                    field_filter_where_and.append(condiction_buffer)
+
+                # Storing condiction value
+                if condiction.value is not None:
+                    if isinstance(condiction.value.__class__, enum.EnumMeta):
+                        if isinstance(condiction.value.value, tuple):
+                            filter_values_map[condiction_alias] = (
+                                condiction.value.value[1]
+                            )
+                        else:
+                            filter_values_map[condiction_alias] = condiction.value.value
                     else:
-                        condiction_alias = ""
-                        condiction_alias_subtituir = ""
-
-                    # Making condiction buffer
-                    if use_table_alias:
-                        condiction_buffer = (
-                            f"t0.{filter_field} {operator} {condiction_alias_subtituir}"
-                        )
-                    else:
-                        condiction_buffer = (
-                            f"{filter_field} {operator} {condiction_alias_subtituir}"
-                        )
-
-                    # Storing field filter where
-                    if operator == "=" or operator == "like" or operator == "ilike":
-                        field_filter_where_or.append(condiction_buffer)
-                    else:
-                        field_filter_where_and.append(condiction_buffer)
-
-                    # Storing condiction value
-                    if condiction.value is not None:
-                        if isinstance(condiction.value.__class__, enum.EnumMeta):
-                            if isinstance(condiction.value.value, tuple):
-                                filter_values_map[
-                                    condiction_alias
-                                ] = condiction.value.value[1]
-                            else:
-                                filter_values_map[
-                                    condiction_alias
-                                ] = condiction.value.value
+                        if (
+                            isinstance(condiction.value, set)
+                            and len(condiction.value) > 1
+                        ):
+                            filter_values_map[condiction_alias] = ", ".join(
+                                str(value) for value in condiction.value
+                            )
                         else:
                             filter_values_map[condiction_alias] = condiction.value
 
-                    if operator == "like" or operator == "ilike":
-                        filter_values_map[
-                            condiction_alias
-                        ] = f"%{filter_values_map[condiction_alias]}%"
+                if operator == "like" or operator == "ilike":
+                    filter_values_map[condiction_alias] = (
+                        f"%{filter_values_map[condiction_alias]}%"
+                    )
 
-                # Formating condictions (with OR)
-                field_filter_where_or = " or ".join(field_filter_where_or)
-                field_filter_where_and = " and ".join(field_filter_where_and)
-                if field_filter_where_or.strip() != "":
-                    field_filter_where_or = f"({field_filter_where_or})"
-                    filters_where.append(field_filter_where_or)
+            # Formating condictions (with OR)
+            field_filter_where_or = " or ".join(field_filter_where_or)
+            field_filter_where_and = " and ".join(field_filter_where_and)
+            if field_filter_where_in:
+                field_filter_where_in = f"{table_alias}.{filter_field} in ({', '.join(field_filter_where_in)})"
+                filters_where.append(field_filter_where_in)
+            if field_filter_where_not_in:
+                field_filter_where_not_in = f"{table_alias}.{filter_field} not in ({', '.join(field_filter_where_not_in)})"
+                filters_where.append(field_filter_where_not_in)
 
-                if field_filter_where_and.strip() != "":
-                    field_filter_where_and = f"({field_filter_where_and})"
-                    filters_where.append(field_filter_where_and)
+            if field_filter_where_or.strip() != "":
+                field_filter_where_or = f"({field_filter_where_or})"
+                filters_where.append(field_filter_where_or)
 
-            # Formating all filters (with AND)
-            filters_where = "\n and ".join(filters_where)
-            if filters_where.strip() != "" and with_and:
-                filters_where = f"and {filters_where}"
+            if field_filter_where_and.strip() != "":
+                field_filter_where_and = f"({field_filter_where_and})"
+                filters_where.append(field_filter_where_and)
+
+        # Formating all filters (with AND)
+        filters_where = "\n and ".join(filters_where)
+        if filters_where.strip() != "" and with_and:
+            filters_where = f"and {filters_where}"
 
         return (filters_where, filter_values_map)
 
@@ -262,6 +301,11 @@ class DAOBase:
         filters: Dict[str, List[Filter]],
         conjunto_type: ConjuntoType = None,
         conjunto_field: str = None,
+        entity_key_field: str = None,
+        entity_id_value: any = None,
+        search_query: str = None,
+        search_fields: List[str] = None,
+        joins_aux: List[JoinAux] = None,
     ) -> List[EntityBase]:
         """
         Returns a paginated entity list.
@@ -278,11 +322,17 @@ class DAOBase:
         order_fields_alias = [f"t0.{i}" for i in order_fields]
 
         # Resolving data to pagination
-        order_map = {field: None for field in order_fields}
+        order_map = {
+            re.sub(r"\basc\b", "", re.sub(r"\bdesc\b", "", field)).strip(): None
+            for field in order_fields
+        }
 
         if after is not None:
             try:
-                after_obj = self.get(entity.get_pk_field(), after, fields)
+                if entity_key_field is None:
+                    after_obj = self.get(entity.get_pk_field(), after, fields)
+                else:
+                    after_obj = self.get(entity_key_field, entity_id_value, fields)
             except NotFoundException as e:
                 raise AfterRecordNotFoundException(
                     f"Identificador recebido no parâmetro after {id}, não encontrado para a entidade {self._entity_class.__name__}."
@@ -290,7 +340,13 @@ class DAOBase:
 
             if after_obj is not None:
                 for field in order_fields:
-                    order_map[field] = getattr(after_obj, field, None)
+                    order_map[
+                        re.sub(r"\basc\b", "", re.sub(r"\bdesc\b", "", field)).strip()
+                    ] = getattr(
+                        after_obj,
+                        re.sub(r"\basc\b", "", re.sub(r"\bdesc\b", "", field)).strip(),
+                        None,
+                    )
 
         # Making default order by clause
         order_by = f"""
@@ -309,13 +365,17 @@ class DAOBase:
                 for of in old_fields:
                     buffer_old_fields += f" and t0.{of} = :{of}"
 
+                field_adjusted = re.sub(
+                    r"\basc\b", "", re.sub(r"\bdesc\b", "", field)
+                ).strip()
+
                 # Making current more than condiction
                 list_page_where.append(
-                    f"({buffer_old_fields} and t0.{field} > :{field})"
+                    f"({buffer_old_fields} and t0.{field_adjusted} {'<' if 'desc' in field else '>'} :{field_adjusted})"
                 )
 
                 # Storing current field as old
-                old_fields.append(field)
+                old_fields.append(field_adjusted)
 
             # Making SQL page condiction
             pagination_where = f"""
@@ -325,6 +385,11 @@ class DAOBase:
                 )
             """
 
+        # Montando o filtro de search (com ilike)
+        search_map, search_where = self._make_search_sql(
+            search_query, search_fields, entity
+        )
+
         # Resolvendo o join de conjuntos (se houver)
         with_conjunto = ""
         fields_conjunto = ""
@@ -341,6 +406,9 @@ class DAOBase:
         # Organizando o where dos filtros
         filters_where, filter_values_map = self._make_filters_sql(filters)
 
+        # Montando a clausula dos fields vindos dos joins
+        sql_join_fields, sql_join = self._make_joins_sql(joins_aux)
+
         # Montando a query em si
         sql = f"""
         {with_conjunto}
@@ -348,15 +416,18 @@ class DAOBase:
 
             {fields_conjunto}
             {self._sql_fields(fields)}
+            {sql_join_fields}
 
         from
             {entity.get_table_name()} as t0
             {join_conjuntos}
+            {sql_join}
 
         where
             true
             {pagination_where}
             {filters_where}
+            {search_where}
 
         order by
             {order_by}
@@ -367,13 +438,153 @@ class DAOBase:
             sql += f"        limit {limit}"
 
         # Making the values dict
-        kwargs = {**order_map, **filter_values_map, **conjunto_map}
+        kwargs = {**order_map, **filter_values_map, **conjunto_map, **search_map}
 
         # Running the SQL query
         resp = self._db.execute_query_to_model(sql, self._entity_class, **kwargs)
 
         return resp
 
+    def _make_joins_sql(self, joins_aux: List[JoinAux] = []):
+        """
+        Método auxiliar, para montar a parte dos campos, e do join propriamente dito,
+        para depois compôr a query principal.
+        """
+
+        if joins_aux is None:
+            return ("", "")
+
+        sql_join_fields = ""
+        sql_join = ""
+        for join_aux in joins_aux:
+            # Ajustando os fields
+            fields_sql = self._sql_fields(
+                fields=join_aux.fields, table_alias=join_aux.alias
+            )
+
+            # Adicionando os fields no SQL geral
+            sql_join_fields = f"{sql_join_fields},\n{fields_sql}"
+
+            # Montando a clausula do join em si
+            join_operator = f"{join_aux.type} join"
+
+            sql_join = f"{sql_join}\n{join_operator} {join_aux.table} as {join_aux.alias} on (t0.{join_aux.self_field} = {join_aux.alias}.{join_aux.other_field})"
+
+        return (sql_join_fields, sql_join)
+
+    def _make_search_sql(
+        self, search_query, search_fields, entity
+    ) -> Tuple[Dict[str, any], str]:
+        """
+        Monta a parte da cláusula where referente ao parâmetro search, bem como o mapa de
+        valores para realizar a pesquisa (passando para a execução da query).
+
+        Retorna uma tupla, onde a primeira posição é o mapa de valores, e a segunda a cláusula sql.
+        """
+
+        search_map = {}
+        search_where = ""
+
+        date_pattern = "(\d\d)/(\d\d)/((\d\d\d\d)|(\d\d))"
+        int_pattern = "(\d+)"
+        float_pattern = "(\d+((,|\.)\d+)?)"
+
+        if search_fields is not None and search_query is not None:
+            search_buffer = "false \n"
+            for search_field in search_fields:
+                search_str = search_query
+
+                entity_field = entity.fields_map.get(search_field)
+                if entity_field is None:
+                    continue
+
+                if (
+                    entity_field.expected_type is datetime.datetime
+                    or entity_field.expected_type is datetime.date
+                ):
+                    # Tratando da busca de datas
+                    received_floats = re.findall(date_pattern, search_str)
+                    cont = -1
+                    for received_float in received_floats:
+                        cont += 1
+
+                        dia = int(received_float[0])
+                        mes = int(received_float[0])
+                        ano = received_float[0]
+                        if len(ano) < 4:
+                            ano = f"20{ano}"
+                        ano = int(ano)
+
+                        data_obj = None
+                        try:
+                            data_obj = datetime.date(ano, mes, dia)
+                        except:
+                            continue
+
+                        search_buffer += (
+                            f" or t0.{search_field} = :shf_{search_field}_{cont} \n"
+                        )
+                        search_map[f"shf_{search_field}_{cont}"] = data_obj
+
+                elif entity_field.expected_type is int:
+                    # Tratando da busca de inteiros
+                    search_str = re.sub(date_pattern, "", search_str)
+
+                    received_floats = re.findall(int_pattern, search_str)
+                    cont = -1
+                    for received_float in received_floats:
+                        cont += 1
+                        valor = int(received_float[0])
+                        valor_min = int(valor * 0.9)
+                        valor_max = int(valor * 1.1)
+
+                        search_buffer += f" or (t0.{search_field} >= :shf_{search_field}_{cont}_min and t0.{search_field} <= :shf_{search_field}_{cont}_max) \n"
+                        search_map[f"shf_{search_field}_{cont}_min"] = valor_min
+                        search_map[f"shf_{search_field}_{cont}_max"] = valor_max
+
+                elif (
+                    entity_field.expected_type is int
+                    or entity_field.expected_type is decimal.Decimal
+                ):
+                    # Tratando da busca de floats e decimais
+                    search_str = re.sub(date_pattern, "", search_str)
+
+                    received_floats = re.findall(float_pattern, search_str)
+                    cont = -1
+                    for received_float in received_floats:
+                        cont += 1
+                        valor = float(received_float[0])
+                        valor_min = valor * 0.9
+                        valor_max = valor * 1.1
+
+                        search_buffer += f" or (t0.{search_field} >= :shf_{search_field}_{cont}_min and t0.{search_field} <= :shf_{search_field}_{cont}_max) \n"
+                        search_map[f"shf_{search_field}_{cont}_min"] = valor_min
+                        search_map[f"shf_{search_field}_{cont}_max"] = valor_max
+
+                elif (
+                    entity_field.expected_type is str
+                    or entity_field.expected_type is uuid
+                ):
+                    # Tratando da busca de strings e UUIDs
+                    cont = -1
+                    for palavra in search_str.split(" "):
+                        if palavra == "":
+                            continue
+
+                        cont += 1
+                        search_buffer += f" or upper(t0.{search_field}) like upper(unaccent(:shf_{search_field}_{cont})) \n"
+                        search_map[f"shf_{search_field}_{cont}"] = (
+                            f"%{unidecode.unidecode(palavra)}%"
+                        )
+
+            search_where = f"""
+            and (
+                {search_buffer}
+            )
+            """
+
+        return search_map, search_where
+
     def _make_conjunto_sql(
         self,
         conjunto_type: ConjuntoType,
@@ -421,7 +632,7 @@ class DAOBase:
                     and est_c0.cadastro = :conjunto_cadastro
                 )
                 group by gemp0.grupoempresarial, gemp0.codigo, est_c0.conjunto
-            )    
+            )
             """
 
         join_conjuntos = f"""
@@ -511,37 +722,43 @@ class DAOBase:
 
         self._db.execute(sql, registro=id)
 
-    def _sql_insert_fields(self) -> str:
+    def _sql_insert_fields(
+        self, entity: EntityBase, sql_read_only_fields: List[str] = []
+    ) -> str:
         """
         Retorna uma tupla com duas partes: (sql_fields, sql_ref_values), onde:
         - sql_fields: Lista de campos a inserir no insert
         - sql_ref_values: Lista das referências aos campos, a inserir no insert (parte values)
         """
 
-        # Creating entity instance
-        entity = self._entity_class()
-
         # Building SQL fields
         fields = [
             f"{k}"
             for k in entity.__dict__
-            if not callable(getattr(entity, k, None)) and not k.startswith("_")
+            if not callable(getattr(entity, k, None))
+            and not k.startswith("_")
+            and (k not in sql_read_only_fields or getattr(entity, k, None) is not None)
         ]
         ref_values = [
             f":{k}"
             for k in entity.__dict__
-            if not callable(getattr(entity, k, None)) and not k.startswith("_")
+            if not callable(getattr(entity, k, None))
+            and not k.startswith("_")
+            and (k not in sql_read_only_fields or getattr(entity, k, None) is not None)
         ]
 
         return (", ".join(fields), ", ".join(ref_values))
 
-    def insert(self, entity: EntityBase):
+
+    def insert(self, entity: EntityBase, sql_read_only_fields: List[str] = []):
         """
         Insere o objeto de entidade "entity" no banco de dados
         """
 
         # Montando as cláusulas dos campos
-        sql_fields, sql_ref_values = self._sql_insert_fields()
+        sql_fields, sql_ref_values = self._sql_insert_fields(
+            entity, sql_read_only_fields
+        )
 
         # Montando a query principal
         sql = f"""
@@ -588,7 +805,38 @@ class DAOBase:
 
         return entity
 
-    def _sql_update_fields(self, entity: EntityBase, ignore_nones: bool = False) -> str:
+    def _sql_upsert_fields(
+        self,
+        entity: EntityBase,
+        ignore_nones: bool = False,
+        sql_read_only_fields: List[str] = [],
+    ) -> str:
+        """
+        Retorna lista com os campos para upsert, no padrão "field = excluded.field"
+        """
+
+        # Building SQL fields
+        fields = [
+            f"{k} = excluded.{k}"
+            for k in entity.__dict__
+            if not callable(getattr(entity, k, None))
+            and not k.startswith("_")
+            and getattr(entity, k) is not None
+            and (ignore_nones and getattr(entity, k) is not None or not ignore_nones)
+            and k not in entity.get_const_fields()
+            and (k != entity.get_pk_field() ) #or getattr(entity, k) is not None)
+            and k not in sql_read_only_fields
+        ]
+
+
+        return ", ".join(fields)
+
+    def _sql_update_fields(
+        self,
+        entity: EntityBase,
+        ignore_nones: bool = False,
+        sql_read_only_fields: List[str] = [],
+    ) -> str:
         """
         Retorna lista com os campos para update, no padrão "field = :field"
         """
@@ -603,6 +851,7 @@ class DAOBase:
                 and getattr(entity, k) is not None
                 and k not in entity.get_const_fields()
                 and (k != entity.get_pk_field() or getattr(entity, k) is not None)
+                and k not in sql_read_only_fields
             ]
         else:
             fields = [
@@ -612,6 +861,7 @@ class DAOBase:
                 and not k.startswith("_")
                 and k not in entity.get_const_fields()
                 and (k != entity.get_pk_field() or getattr(entity, k) is not None)
+                and k not in sql_read_only_fields
             ]
 
         return ", ".join(fields)
@@ -623,16 +873,15 @@ class DAOBase:
         entity: EntityBase,
         filters: Dict[str, List[Filter]],
         partial_update: bool = False,
+        sql_read_only_fields: List[str] = [],
+        upsert: bool = False
     ):
         """
         Atualiza o objeto de entidade "entity" no banco de dados
         """
 
-        # Montando a cláusula dos campos
-        sql_fields = self._sql_update_fields(entity, partial_update)
-
         # Organizando o where dos filtros
-        filters_where, filter_values_map = self._make_filters_sql(filters, True, False)
+        filters_where, filter_values_map = self._make_filters_sql(filters, True, True)
 
         # # CUIDADO PARA NÂO ATUALIZAR O QUE NÃO DEVE
         # if filters_where is None or filters_where.strip() == "":
@@ -640,17 +889,63 @@ class DAOBase:
         #         f"{self._entity_class.__name__} não encontrado. Filtros: {filters}"
         #     )
 
-        # Montando a query principal
-        sql = f"""
-        update {entity.get_table_name()} set
+        # Montando cláusula upsert
+        if upsert:
 
-            {sql_fields}
+            # Montando as cláusulas dos campos
+            sql_fields, sql_ref_values = self._sql_insert_fields(
+                entity, sql_read_only_fields
+            )
 
-        where
-            true
-            and {key_field} = :candidate_key_value
-            {filters_where}
-        """
+            sql_upsert_fields = self._sql_upsert_fields(
+                entity, partial_update, sql_read_only_fields
+            )
+
+            conflict_fields = f"{entity.get_pk_field()}{',' + ','.join(filters.keys()) if filters else ''}"
+
+            conflict_rules = f"""
+            ON CONFLICT ({conflict_fields}) DO
+            UPDATE
+            SET
+                {sql_upsert_fields}
+
+                """
+
+            # Montando a query principal
+            sql = f"""
+            insert into {entity.get_table_name()} as t0 (
+
+                {sql_fields}
+
+            ) values (
+
+                {sql_ref_values}
+
+            )
+            {conflict_rules}
+            where
+                true
+                and t0.{key_field} = :candidate_key_value
+                {filters_where}
+            """
+        else:
+
+            # Montando a cláusula dos campos
+            sql_fields = self._sql_update_fields(
+                entity, partial_update, sql_read_only_fields
+            )
+
+            # Montando a query principal
+            sql = f"""
+            update {entity.get_table_name()} set
+
+                {sql_fields}
+
+            where
+                true
+                and {key_field} = :candidate_key_value
+                {filters_where}
+            """
 
         # Montando as cláusulas returning
         returning_fields = entity.get_update_returning_fields()
@@ -768,4 +1063,4 @@ class DAOBase:
 
             return True
         except ValueError:
-            return False
+            return False
\ No newline at end of file
