@@ -1,4 +1,3 @@
-import enum
 import typing
 
 from nsj_rest_lib.dto.dto_base import DTOBase
@@ -12,14 +11,15 @@ class DTOListField:
     def __init__(
         self,
         dto_type: DTOBase,
-        entity_type: EntityBase,
-        related_entity_field: str,
+        entity_type: EntityBase = None,
+        related_entity_field: str = None,
         not_null: bool = False,
         min: int = None,
         max: int = None,
         validator: typing.Callable = None,
         dto_post_response_type: DTOBase = None,
         relation_key_field: str = None,
+        service_name: str = None,
     ):
         """
         -----------
@@ -35,6 +35,8 @@ class DTOListField:
         related_entity_field: Fields, from related entity, used for relation in database.
         relation_key_field: Nome do campo, no DTO corrente, utilizado como chave de apontamento no relacionamento
             (isso é, campo para o qual a entidade, do lado N, aponta via FK).
+        service_name: Nome do serviço customizado, caso se deseje que as operações sobre esse tipo de lista se façam
+            de um modo customizado (e não usando o service_base do próprio RestLib).
         """
         self.name = None
         self.dto_type = dto_type
@@ -46,6 +48,7 @@ class DTOListField:
         self.validator = validator
         self.dto_post_response_type = dto_post_response_type
         self.relation_key_field = relation_key_field
+        self.service_name = service_name
 
         self.storage_name = f"_{self.__class__.__name__}#{self.__class__._ref_counter}"
         self.__class__._ref_counter += 1
@@ -54,8 +57,11 @@ class DTOListField:
         if self.dto_type is None:
             raise DTOListFieldConfigException("type parameter must be not None.")
 
-        if self.entity_type is None:
-            raise DTOListFieldConfigException("entity_type parameter must be not None.")
+        if service_name is None:
+            if self.entity_type is None:
+                raise DTOListFieldConfigException(
+                    "entity_type parameter must be not None."
+                )
 
     def __get__(self, instance, owner):
         if instance is None:
@@ -64,10 +70,17 @@ class DTOListField:
             return instance.__dict__[self.storage_name]
 
     def __set__(self, instance, value):
-        if self.validator is None:
-            value = self.validate(value)
-        else:
-            value = self.validator(value)
+        try:
+            if self.validator is None:
+                value = self.validate(value)
+            else:
+                value = self.validator(value)
+        except ValueError:
+            if not (
+                "escape_validator" in instance.__dict__
+                and instance.__dict__["escape_validator"] is True
+            ):
+                raise
 
         # Preenchendo os campos de particionanmento, se necessário (normalmente: tenant e grupo_empresarial)
         self.set_partition_fields(instance, value)
@@ -128,13 +141,13 @@ class DTOListField:
             )
 
         # Checking min constraint
-        if self.min is not None and len(value) < self.min:
+        if self.min is not None and (value is None or len(value) < self.min):
             raise ValueError(
                 f"A lista {self.storage_name} deve ter mais do que {self.min} itens. Valor recebido: {value}."
             )
 
-        # Checking min constraint
-        if self.max is not None and len(value) > self.max:
+        # Checking max constraint
+        if self.max is not None and value is not None and len(value) > self.max:
             raise ValueError(
                 f"A lista {self.storage_name} deve ter menos do que {self.max} itens. Valor recebido: {value}."
             )
