@@ -18,6 +18,10 @@ class DTOBase(abc.ABC):
     list_fields_map: dict = {}
     left_join_fields_map: dict = {}
     left_join_fields_map_to_query: dict = {}
+    sql_join_fields_map: dict = {}
+    sql_join_fields_map_to_query: dict = {}
+    sql_read_only_fields: list = []
+    object_fields_map: list = []
     field_filters_map: Dict[str, DTOFieldFilter]
     # TODO Refatorar para suportar PK composto
     pk_field: str
@@ -27,12 +31,15 @@ class DTOBase(abc.ABC):
     escape_validator: bool
     uniques: Dict[str, Set[str]]
     candidate_keys: List[str]
+    search_fields: Set[str]
 
     def __init__(
         self,
         entity: Union[EntityBase, dict] = None,
         escape_validator: bool = False,
         generate_default_pk_value: bool = True,
+        validate_read_only: bool = False,
+        kwargs_as_entity: bool = False,
         **kwargs,
     ) -> None:
         super().__init__()
@@ -51,29 +58,74 @@ class DTOBase(abc.ABC):
         # Setando os campos registrados como fields simples
         for field in self.__class__.fields_map:
             # Recuperando a configuração do campo
-            left_join_dto_field = self.__class__.fields_map[field]
+            aux_dto_field = self.__class__.fields_map[field]
+
+            if (
+                validate_read_only
+                and aux_dto_field.read_only
+                and kwargs.get(field, None) is not None
+            ):
+                raise ValueError(f"O campo {field} não pode ser preenchido.")
 
             # Tratando do valor default
             if (
-                left_join_dto_field.default_value is not None
+                aux_dto_field.default_value is not None
                 and kwargs.get(field, None) is None
-                and (not left_join_dto_field.pk or generate_default_pk_value)
+                and (not aux_dto_field.pk or generate_default_pk_value)
             ):
-                default_value = left_join_dto_field.default_value
-                if callable(left_join_dto_field.default_value):
-                    default_value = left_join_dto_field.default_value()
+                default_value = aux_dto_field.default_value
+                if callable(aux_dto_field.default_value):
+                    default_value = aux_dto_field.default_value()
                 kwargs[field] = default_value
 
             # Verificando se é preciso converter o nome do field para o nome correspondente no Entity
             # E, se será preciso aplicar alguma conversão customizada (para trazer o valor do entity para o DTO)
             entity_field = field
-            if entity is not None:
-                if left_join_dto_field.entity_field is not None:
-                    entity_field = left_join_dto_field.entity_field
+            if entity is not None or kwargs_as_entity:
+                if aux_dto_field.entity_field is not None:
+                    entity_field = aux_dto_field.entity_field
 
                 # Verificando se o campo carece de conversão customizada
-                if left_join_dto_field.convert_from_entity is not None:
-                    fields_converted = left_join_dto_field.convert_from_entity(
+                if aux_dto_field.convert_from_entity is not None:
+                    fields_converted = aux_dto_field.convert_from_entity(
+                        kwargs[entity_field], kwargs
+                    )
+                    if field not in fields_converted:
+                        setattr(self, field, None)
+
+                    for converted_key in fields_converted:
+                        setattr(self, converted_key, fields_converted[converted_key])
+
+                    continue
+
+            # Atribuindo o valor à propriedade do DTO
+            if entity_field in kwargs:
+                setattr(self, field, kwargs[entity_field])
+            else:
+                setattr(self, field, None)
+
+        # Setando os campos registrados como fields de join por meio da consulta SQL
+        for field in self.__class__.sql_join_fields_map:
+            # Recuperando a configuração do campo
+            aux_dto_field = self.__class__.sql_join_fields_map[field]
+
+            # Verificando a conversão do nome do field para o nome correspondente no Entity (no DTO Relacionado)
+            # E, se será preciso aplicar alguma conversão customizada (para trazer o valor do entity para o DTO)
+            entity_field = field
+            if entity is not None or kwargs_as_entity:
+                if (
+                    not aux_dto_field.related_dto_field
+                    in aux_dto_field.dto_type.fields_map
+                ):
+                    continue
+
+                entity_field = aux_dto_field.dto_type.fields_map[
+                    aux_dto_field.related_dto_field
+                ].get_entity_field_name()
+
+                # Verificando se o campo carece de conversão customizada
+                if aux_dto_field.convert_from_entity is not None:
+                    fields_converted = aux_dto_field.convert_from_entity(
                         kwargs[entity_field], kwargs
                     )
                     if field not in fields_converted:
@@ -93,7 +145,7 @@ class DTOBase(abc.ABC):
         # Setando os campos registrados como fields left join
         for field in self.__class__.left_join_fields_map:
             # Recuperando a configuração do campo
-            left_join_dto_field = self.__class__.left_join_fields_map[field]
+            aux_dto_field = self.__class__.left_join_fields_map[field]
 
             # Atribuindo o valor à propriedade do DTO
             if field in kwargs:
@@ -101,6 +153,17 @@ class DTOBase(abc.ABC):
             else:
                 setattr(self, field, None)
 
+        # Setando os campos registrados como fields object
+        for field in self.__class__.object_fields_map:
+            # Recuperando a configuração do campo
+            aux_dto_field = self.__class__.object_fields_map[field]
+
+            # Atribuindo o valor à propriedade do DTO
+            if field in kwargs:
+                setattr(self, field, aux_dto_field.expected_type(**kwargs[field]))
+            else:
+                setattr(self, field, None)
+
         # Setando os campos registrados como fields de lista
         if entity is None:
             for field in self.__class__.list_fields_map:
@@ -341,6 +404,13 @@ class DTOBase(abc.ABC):
 
             result[field] = getattr(self, field)
 
+        # Converting sql join fields
+        for field in self.sql_join_fields_map:
+            if not field in fields["root"]:
+                continue
+
+            result[field] = getattr(self, field)
+
         # Converting left join fields
         for field in self.left_join_fields_map:
             if not field in fields["root"]:
@@ -353,13 +423,13 @@ class DTOBase(abc.ABC):
             if not field in fields["root"]:
                 continue
 
-            # Criando o mapa de fileds para a entidade aninhada
+            # Criando o mapa de fields para a entidade aninhada
             internal_fields = None
             if field in fields:
                 internal_fields = {"root": fields[field]}
 
             # Recuperando o valor do atributo
-            value = getattr(self, field)
+            value = getattr(self, field, None)
             if value is None:
                 value = []
 
