@@ -1,5 +1,5 @@
-import uuid
 import copy
+import uuid
 
 from typing import Any, Callable, Dict, List, Set, Tuple
 
@@ -24,6 +24,10 @@ from nsj_rest_lib.exception import (
 )
 from nsj_rest_lib.injector_factory_base import NsjInjectorFactoryBase
 from nsj_rest_lib.validator.validate_data import validate_uuid
+from nsj_rest_lib.util.join_aux import JoinAux
+from nsj_rest_lib.util.type_validator_util import TypeValidatorUtil
+
+from nsj_gcf_utils.db_adapter2 import DBAdapter2
 
 
 class ServiceBase:
@@ -46,6 +50,30 @@ class ServiceBase:
         self._created_by_property = "criado_por"
         self._updated_by_property = "atualizado_por"
 
+    @staticmethod
+    def construtor1(
+        db_adapter: DBAdapter2,
+        dao: DAOBase,
+        dto_class: DTOBase,
+        entity_class: EntityBase,
+        dto_post_response_class: DTOBase = None,
+    ):
+        """
+        Esse construtor alternativo, evita a necessidade de passar um InjectorFactory,
+        pois esse só é usado (internamente) para recuperar um db_adapter.
+
+        Foi feito para não gerar breaking change de imediato (a ideia porém é, no futuro,
+        gerar um breaking change).
+        """
+
+        class FakeInjectorFactory:
+            def db_adapter():
+                return db_adapter
+
+        return ServiceBase(
+            FakeInjectorFactory(), dao, dto_class, entity_class, dto_post_response_class
+        )
+
     def get(
         self,
         id: str,
@@ -73,6 +101,9 @@ class ServiceBase:
             partition_fields,
         )
 
+        # Resolvendo os joins
+        joins_aux = self._resolve_sql_join_fields(fields["root"], entity_filters)
+
         # Recuperando a entity
         entity = self._dao.get(
             entity_key_field,
@@ -81,6 +112,7 @@ class ServiceBase:
             entity_filters,
             conjunto_type=self._dto_class.conjunto_type,
             conjunto_field=self._dto_class.conjunto_field,
+            joins_aux=joins_aux,
         )
 
         # Convertendo para DTO
@@ -166,9 +198,14 @@ class ServiceBase:
             f"Não foi possível identificar o ID recebido com qualquer das chaves candidatas reconhecidas. Valor recebido: {id_value}."
         )
 
-    def _convert_to_entity_fields(self, fields: Set[str]) -> List[str]:
+    def _convert_to_entity_fields(
+        self,
+        fields: Set[str],
+        dto_class=None,
+        entity_class=None,
+    ) -> List[str]:
         """
-        Convert a list of fields names to a list of entity fiedls names.
+        Convert a list of fields names to a list of entity fields names.
         """
 
         if fields is None:
@@ -176,15 +213,22 @@ class ServiceBase:
 
         # TODO Refatorar para não precisar deste objeto só por conta das propriedades da classe
         # (um decorator na classe, poderia armazenar os fields na mesma, como é feito no DTO)
-        entity = self._entity_class()
+        if entity_class is None:
+            entity = self._entity_class()
+        else:
+            entity = entity_class()
+
+        # Resolvendo a classe padrão de DTO
+        if dto_class is None:
+            dto_class = self._dto_class
 
         entity_fields = []
         for field in fields:
             # Skipping not DTO fields
-            if not (field in self._dto_class.fields_map):
+            if not (field in dto_class.fields_map):
                 continue
 
-            entity_field_name = self._convert_to_entity_field(field)
+            entity_field_name = self._convert_to_entity_field(field, dto_class)
             # Skipping not Entity fields
             if not (entity_field_name in entity.__dict__):
                 continue
@@ -193,14 +237,22 @@ class ServiceBase:
 
         return entity_fields
 
-    def _convert_to_entity_field(self, field: str) -> str:
+    def _convert_to_entity_field(
+        self,
+        field: str,
+        dto_class=None,
+    ) -> str:
         """
         Convert a field name to a entity field name.
         """
 
+        # Resolvendo a classe padrão de DTO
+        if dto_class is None:
+            dto_class = self._dto_class
+
         entity_field_name = field
-        if self._dto_class.fields_map[field].entity_field is not None:
-            entity_field_name = self._dto_class.fields_map[field].entity_field
+        if dto_class.fields_map[field].entity_field is not None:
+            entity_field_name = dto_class.fields_map[field].entity_field
 
         return entity_field_name
 
@@ -215,88 +267,214 @@ class ServiceBase:
         if filters is None:
             return None
 
+        # Construindo um novo dict de filtros para controle
+        aux_filters = copy.deepcopy(filters)
+        fist_run = True
+
+        # Dicionário para guardar os filtros convertidos
         entity_filters = {}
-        for filter in filters:
-            is_entity_filter = False
-            is_conjunto_filter = False
-            dto_field = None
 
-            if filter in self._dto_class.field_filters_map:
-                # Retrieving filter config
-                field_filter = self._dto_class.field_filters_map[filter]
-                aux = self._dto_class.field_filters_map[filter].field_name
-                dto_field = self._dto_class.fields_map[aux]
-            elif filter == self._dto_class.conjunto_field:
-                is_conjunto_filter = True
-                dto_field = self._dto_class.fields_map[self._dto_class.conjunto_field]
-            elif filter in self._dto_class.fields_map:
-                # Creating filter config to a DTOField (equals operator)
-                field_filter = DTOFieldFilter(filter)
-                field_filter.set_field_name(filter)
-                dto_field = self._dto_class.fields_map[filter]
-            # TODO Refatorar para usar um mapa de fields do entity
-            elif filter in self._entity_class().__dict__:
-                is_entity_filter = True
-            else:
-                # Ignoring not declared filters (or filter for not existent DTOField)
-                continue
+        # Iterando enquanto houver filtros recebidos, ou derivalos a partir dos filter_aliases
+        while len(aux_filters) > 0:
+            new_filters = {}
 
-            # Resolving entity field name (to filter)
-            if not is_entity_filter and not is_conjunto_filter:
-                entity_field_name = self._convert_to_entity_field(
-                    field_filter.field_name
-                )
-            else:
-                entity_field_name = filter
+            for filter in aux_filters:
+                is_entity_filter = False
+                is_conjunto_filter = False
+                is_sql_join_filter = False
+                dto_field = None
+                dto_sql_join_field = None
+                table_alias = None
 
-            # Creating entity filters (one for each value - separated by comma)
-            if isinstance(filters[filter], str):
-                values = filters[filter].split(",")
-            else:
-                values = [filters[filter]]
+                # Recuperando os valores passados nos filtros
+                if isinstance(aux_filters[filter], str):
+                    values = aux_filters[filter].split(",")
+                else:
+                    values = [aux_filters[filter]]
 
-            for value in values:
-                if isinstance(value, str):
-                    value = value.strip()
+                if len(values) <= 0:
+                    # Se não houver valor a filtrar, o filtro é apenas ignorado
+                    continue
 
-                # Convertendo os valores para o formato esperado no entity
-                if not is_entity_filter:
-                    converted_values = self._dto_class.custom_convert_value_to_entity(
-                        value,
-                        dto_field,
-                        entity_field_name,
-                        False,
-                        filters,
+                # Identificando o tipo de filtro passado
+                if (
+                    self._dto_class.filter_aliases is not None
+                    and filter in self._dto_class.filter_aliases
+                    and fist_run
+                ):
+                    # Verificando se é um alias para outros filtros (o alias aponta para outros filtros,
+                    # de acordo com o tipo do dado recebido)
+                    filter_aliases = self._dto_class.filter_aliases[filter]
+
+                    # Iterando os tipos definidos para o alias, e verificando se casam com o tipo recebido
+                    for type_alias in filter_aliases:
+                        relative_field = filter_aliases[type_alias]
+
+                        # Esse obj abaixo é construído artificialmente, com os campos esperados no método validate
+                        # Se o validate mudar, tem que refatorar aqui:
+                        class OBJ:
+                            def __init__(self) -> None:
+                                self.expected_type = None
+                                self.storage_name = None
+
+                        obj = OBJ()
+                        obj.expected_type = type_alias
+                        obj.storage_name = filter
+
+                        # Verificando se é possível converter o valor recebido para o tipo definido no alias do filtro
+                        try:
+                            TypeValidatorUtil.validate(obj, values[0])
+                            convertido = True
+                        except:
+                            convertido = False
+
+                        if convertido:
+                            # Se conseguiu converter para o tipo correspondente, se comportará exatamente como um novo
+                            # filtro, porém como se tivesse sido passado para o campo correspondente ao tipo:
+                            if relative_field not in new_filters:
+                                new_filters[relative_field] = aux_filters[filter]
+                            else:
+                                new_filters[relative_field] = (
+                                    f"{new_filters[relative_field]},{aux_filters[filter]}"
+                                )
+                            break
+
+                        else:
+                            # Se não encontrar conseguir converter (até o final, será apenas ignorado)
+                            pass
+
+                    continue
+
+                elif filter in self._dto_class.field_filters_map:
+                    # Retrieving filter config
+                    field_filter = self._dto_class.field_filters_map[filter]
+                    aux = self._dto_class.field_filters_map[filter].field_name
+                    dto_field = self._dto_class.fields_map[aux]
+
+                elif filter == self._dto_class.conjunto_field:
+                    is_conjunto_filter = True
+                    dto_field = self._dto_class.fields_map[
+                        self._dto_class.conjunto_field
+                    ]
+
+                elif filter in self._dto_class.fields_map:
+                    # Creating filter config to a DTOField (equals operator)
+                    field_filter = DTOFieldFilter(filter)
+                    field_filter.set_field_name(filter)
+                    dto_field = self._dto_class.fields_map[filter]
+
+                elif filter in self._dto_class.sql_join_fields_map:
+                    # Creating filter config to a DTOSQLJoinField (equals operator)
+                    is_sql_join_filter = True
+                    field_filter = DTOFieldFilter(filter)
+                    field_filter.set_field_name(filter)
+                    dto_sql_join_field = self._dto_class.sql_join_fields_map[filter]
+                    dto_field = dto_sql_join_field.dto_type.fields_map[
+                        dto_sql_join_field.related_dto_field
+                    ]
+
+                    # Procurando o table alias
+                    for join_query_key in self._dto_class.sql_join_fields_map_to_query:
+                        join_query = self._dto_class.sql_join_fields_map_to_query[
+                            join_query_key
+                        ]
+                        if filter in join_query.fields:
+                            table_alias = join_query.sql_alias
+
+                # TODO Refatorar para usar um mapa de fields do entity
+                elif filter in self._entity_class().__dict__:
+                    is_entity_filter = True
+
+                else:
+                    # Ignoring not declared filters (or filter for not existent DTOField)
+                    continue
+
+                # Resolving entity field name (to filter)
+                if (
+                    not is_entity_filter
+                    and not is_conjunto_filter
+                    and not is_sql_join_filter
+                ):
+                    entity_field_name = self._convert_to_entity_field(
+                        field_filter.field_name
                     )
-                    if len(converted_values) <= 0:
-                        value = self._dto_class.convert_value_to_entity(
+                elif is_sql_join_filter:
+                    # TODO Verificar se precisa de um if dto_sql_join_field.related_dto_field in dto_sql_join_field.dto_type.fields_map
+                    entity_field_name = dto_sql_join_field.dto_type.fields_map[
+                        dto_sql_join_field.related_dto_field
+                    ].get_entity_field_name()
+                else:
+                    entity_field_name = filter
+
+                # Creating entity filters (one for each value - separated by comma)
+                for value in values:
+                    if isinstance(value, str):
+                        value = value.strip()
+
+                    # Resolvendo as classes de DTO e Entity
+                    aux_dto_class = self._dto_class
+                    aux_entity_class = self._entity_class
+
+                    if is_sql_join_filter:
+                        aux_dto_class = dto_sql_join_field.dto_type
+                        aux_entity_class = dto_sql_join_field.entity_type
+
+                    # Convertendo os valores para o formato esperado no entity
+                    if not is_entity_filter and not is_sql_join_filter:
+                        converted_values = aux_dto_class.custom_convert_value_to_entity(
                             value,
                             dto_field,
+                            entity_field_name,
                             False,
-                            self._entity_class,
+                            aux_filters,
                         )
-                        converted_values = {entity_field_name: value}
-                else:
-                    converted_values = {entity_field_name: value}
+                        if len(converted_values) <= 0:
+                            value = aux_dto_class.convert_value_to_entity(
+                                value,
+                                dto_field,
+                                False,
+                                aux_entity_class,
+                            )
+                            converted_values = {entity_field_name: value}
 
-                # Tratando cada valor convertido
-                for entity_field in converted_values:
-                    converted_value = converted_values[entity_field]
-
-                    if not is_entity_filter and not is_conjunto_filter:
-                        entity_filter = Filter(field_filter.operator, converted_value)
                     else:
-                        entity_filter = Filter(FilterOperator.EQUALS, converted_value)
+                        converted_values = {entity_field_name: value}
 
-                    # Storing filter in dict
-                    filter_list = entity_filters.setdefault(entity_field, [])
-                    filter_list.append(entity_filter)
+                    # Tratando cada valor convertido
+                    for entity_field in converted_values:
+                        converted_value = converted_values[entity_field]
+
+                        if (
+                            not is_entity_filter
+                            and not is_conjunto_filter
+                            and not is_sql_join_filter
+                        ):
+                            entity_filter = Filter(
+                                field_filter.operator, converted_value
+                            )
+                        elif is_sql_join_filter:
+                            entity_filter = Filter(
+                                field_filter.operator, converted_value, table_alias
+                            )
+                        else:
+                            entity_filter = Filter(
+                                FilterOperator.EQUALS, converted_value
+                            )
+
+                        # Storing filter in dict
+                        filter_list = entity_filters.setdefault(entity_field, [])
+                        filter_list.append(entity_filter)
+
+            # Ajustando as variáveis de controle
+            fist_run = False
+            aux_filters = {}
+            aux_filters.update(new_filters)
 
         return entity_filters
 
     def _resolving_fields(self, fields: Dict[str, Set[str]]) -> Dict[str, Set[str]]:
         """
-        Varifica os fields recebidos, garantindo que os campos de resumo serão considerados.
+        Verifica os fields recebidos, garantindo que os campos de resumo serão considerados.
         """
 
         # Resolving fields
@@ -308,6 +486,90 @@ class ServiceBase:
 
         return result
 
+    def filter_list(self, filters: Dict[str, Any]):
+        return self.list(
+            None,
+            None,
+            {"root": set()},
+            None,
+            filters,
+        )
+
+    def _resolve_sql_join_fields(
+        self,
+        fields: Set[str],
+        entity_filters: Dict[str, List[Filter]],
+    ) -> List[JoinAux]:
+        """
+        Analisa os campos de jooin solicitados, e monta uma lista de objetos
+        para auxiliar o DAO na construção da query
+        """
+
+        # Criando o objeto de retorno
+        joins_aux: List[JoinAux] = []
+
+        # Iterando os campos de join configurados, mas só considerando os solicitados (ou de resumo)
+        for join_field_map_to_query_key in self._dto_class.sql_join_fields_map_to_query:
+            join_field_map_to_query = self._dto_class.sql_join_fields_map_to_query[
+                join_field_map_to_query_key
+            ]
+
+            used_join_fields = set()
+
+            # Verificando se um dos campos desse join será usado
+            for join_field in join_field_map_to_query.fields:
+                # Recuperando o nome do campo, na entity
+                entity_join_field = join_field_map_to_query.related_dto.fields_map[
+                    self._dto_class.sql_join_fields_map[join_field].related_dto_field
+                ].get_entity_field_name()
+
+                if join_field in fields or entity_join_field in entity_filters:
+                    relate_join_field = self._dto_class.sql_join_fields_map[
+                        join_field
+                    ].related_dto_field
+                    used_join_fields.add(relate_join_field)
+
+            # Pulando esse join (se não for usado)
+            if len(used_join_fields) <= 0:
+                continue
+
+            # Construindo o objeto auxiliar do join
+            join_aux = JoinAux()
+
+            # Resolvendo os nomes dos fields da entidade relacionada
+            join_entity_fields = self._convert_to_entity_fields(
+                fields=used_join_fields,
+                dto_class=join_field_map_to_query.related_dto,
+                entity_class=join_field_map_to_query.related_entity,
+            )
+
+            join_aux.fields = join_entity_fields
+
+            # Resolvendo tabela, tipo de join e alias
+            other_entity = join_field_map_to_query.related_entity()
+            join_aux.table = other_entity.get_table_name()
+            join_aux.type = join_field_map_to_query.join_type
+            join_aux.alias = join_field_map_to_query.sql_alias
+
+            # Resovendo os campos usados no join
+            if (
+                join_field_map_to_query.entity_relation_owner
+                == EntityRelationOwner.SELF
+            ):
+                join_aux.self_field = self._dto_class.fields_map[
+                    join_field_map_to_query.relation_field
+                ].get_entity_field_name()
+                join_aux.other_field = other_entity.get_pk_field()
+            else:
+                join_aux.self_field = self._entity_class().get_pk_field()
+                join_aux.other_field = join_field_map_to_query.related_dto.fields_map[
+                    join_field_map_to_query.relation_field
+                ].get_entity_field_name()
+
+            joins_aux.append(join_aux)
+
+        return joins_aux
+
     def list(
         self,
         after: uuid.UUID,
@@ -315,6 +577,7 @@ class ServiceBase:
         fields: Dict[str, Set[str]],
         order_fields: List[str],
         filters: Dict[str, Any],
+        search_query: str = None,
     ) -> List[DTOBase]:
         # Resolving fields
         fields = self._resolving_fields(fields)
@@ -334,6 +597,24 @@ class ServiceBase:
 
         entity_filters = self._create_entity_filters(all_filters)
 
+        # Tratando dos campos a serem enviados ao DAO para uso do search (se necessário)
+        search_fields = None
+        if self._dto_class.search_fields is not None:
+            search_fields = self._convert_to_entity_fields(
+                self._dto_class.search_fields
+            )
+
+        # Resolve o campo de chave sendo utilizado
+        entity_key_field, entity_id_value = (None, None)
+        if after is not None:
+            entity_key_field, entity_id_value = self._resolve_field_key(
+                after,
+                filters,
+            )
+
+        # Resolvendo os joins
+        joins_aux = self._resolve_sql_join_fields(fields["root"], entity_filters)
+
         # Retrieving from DAO
         entity_list = self._dao.list(
             after,
@@ -343,6 +624,11 @@ class ServiceBase:
             entity_filters,
             conjunto_type=self._dto_class.conjunto_type,
             conjunto_field=self._dto_class.conjunto_field,
+            entity_key_field=entity_key_field,
+            entity_id_value=entity_id_value,
+            search_query=search_query,
+            search_fields=search_fields,
+            joins_aux=joins_aux,
         )
 
         # Convertendo para uma lista de DTOs
@@ -377,15 +663,20 @@ class ServiceBase:
             for master_dto_attr, list_field in self._dto_class.list_fields_map.items():
                 if master_dto_attr in fields["root"]:
                     # Getting service instance
-                    # TODO Refatorar para suportar services customizados
-                    service = ServiceBase(
-                        self._injector_factory,
-                        DAOBase(
-                            self._injector_factory.db_adapter(), list_field.entity_type
-                        ),
-                        list_field.dto_type,
-                        list_field.entity_type,
-                    )
+                    if list_field.service_name is not None:
+                        service = self._injector_factory.get_service_by_name(
+                            list_field.service_name
+                        )
+                    else:
+                        service = ServiceBase(
+                            self._injector_factory,
+                            DAOBase(
+                                self._injector_factory.db_adapter(),
+                                list_field.entity_type,
+                            ),
+                            list_field.dto_type,
+                            list_field.entity_type,
+                        )
 
                     # Checking if pk_field exists
                     if self._dto_class.pk_field is None:
@@ -403,12 +694,15 @@ class ServiceBase:
                     if list_field.relation_key_field is not None:
                         relation_key_field = list_field.relation_key_field
 
+                    # Getting value to filter related list
+                    relation_filter_value = getattr(dto, relation_key_field)
+                    if relation_filter_value is None:
+                        # If None, there is no related objects. So, set empty list and continue.
+                        setattr(dto, master_dto_attr, [])
+                        continue
+
                     # Making filter to relation
-                    filters = {
-                        list_field.related_entity_field: getattr(
-                            dto, relation_key_field
-                        )
-                    }
+                    filters = {list_field.related_entity_field: relation_filter_value}
 
                     # Tratando campos de particionamento
                     for field in self._dto_class.partition_fields:
@@ -448,6 +742,41 @@ class ServiceBase:
             custom_after_insert=custom_after_insert,
         )
 
+    def insert_list(
+        self,
+        dtos: List[DTOBase],
+        aditional_filters: Dict[str, Any] = None,
+        custom_before_insert: Callable = None,
+        custom_after_insert: Callable = None,
+    ) -> List[DTOBase]:
+
+        _lst_return = []
+        try:
+            self._dao.begin()
+
+            for dto in dtos:
+                _return_object = self._save(
+                    insert=True,
+                    dto=dto,
+                    manage_transaction=False,
+                    partial_update=False,
+                    aditional_filters=aditional_filters,
+                    custom_before_insert=custom_before_insert,
+                    custom_after_insert=custom_after_insert,
+                )
+
+                if _return_object is not None:
+                    _lst_return.append(_return_object)
+
+        except:
+            self._dao.rollback()
+            raise
+        finally:
+            self._dao.commit()
+
+        return _lst_return
+
+
     def update(
         self,
         dto: DTOBase,
@@ -455,6 +784,7 @@ class ServiceBase:
         aditional_filters: Dict[str, Any] = None,
         custom_before_update: Callable = None,
         custom_after_update: Callable = None,
+        upsert: bool = False
     ) -> DTOBase:
         return self._save(
             insert=False,
@@ -465,8 +795,46 @@ class ServiceBase:
             aditional_filters=aditional_filters,
             custom_before_update=custom_before_update,
             custom_after_update=custom_after_update,
+            upsert=upsert
         )
 
+    def update_list(
+        self,
+        dtos: List[DTOBase],
+        aditional_filters: Dict[str, Any] = None,
+        custom_before_update: Callable = None,
+        custom_after_update: Callable = None,
+        upsert: bool = False
+    ) -> List[DTOBase]:
+
+        _lst_return = []
+        try:
+            self._dao.begin()
+
+            for dto in dtos:
+                _return_object = self._save(
+                    insert=False,
+                    dto=dto,
+                    manage_transaction=True,
+                    partial_update=False,
+                    id=getattr(dto, dto.pk_field),
+                    aditional_filters=aditional_filters,
+                    custom_before_update=custom_before_update,
+                    custom_after_update=custom_after_update,
+                    upsert=upsert
+                )
+
+                if _return_object is not None:
+                    _lst_return.append(_return_object)
+
+        except:
+            self._dao.rollback()
+            raise
+        finally:
+            self._dao.commit()
+
+        return _lst_return
+
     def partial_update(
         self,
         dto: DTOBase,
@@ -519,6 +887,7 @@ class ServiceBase:
         custom_before_update: Callable = None,
         custom_after_insert: Callable = None,
         custom_after_update: Callable = None,
+        upsert: bool = False
     ) -> DTOBase:
         try:
             # Guardando um ponteiro para o DTO recebido
@@ -527,11 +896,14 @@ class ServiceBase:
             if manage_transaction:
                 self._dao.begin()
 
+            old_dto = None
             # Recuperando o DTO antes da gravação (apenas se for update, e houver um custom_after_update)
-            if not insert and (
-                custom_after_update is not None or custom_before_update is not None
-            ):
+            if not insert and not upsert:
                 old_dto = self._retrieve_old_dto(dto, id, aditional_filters)
+                setattr(dto, dto.pk_field, getattr(old_dto, dto.pk_field))
+
+            if not insert and upsert:
+                old_dto = dto
 
             if custom_before_insert:
                 received_dto = copy.deepcopy(dto)
@@ -595,12 +967,6 @@ class ServiceBase:
             else:
                 aditional_entity_filters = {}
 
-            # Resolve o campo de chave sendo utilizado
-            entity_key_field, entity_id_value = self._resolve_field_key(
-                id,
-                dto.__dict__,
-            )
-
             # Validando as uniques declaradas
             for unique in self._dto_class.uniques:
                 unique = self._dto_class.uniques[unique]
@@ -609,8 +975,7 @@ class ServiceBase:
                     entity,
                     aditional_entity_filters,
                     unique,
-                    entity_key_field,
-                    entity_id_value,
+                    old_dto,
                 )
 
             # Invocando o DAO
@@ -623,7 +988,7 @@ class ServiceBase:
                     )
 
                 # Inserindo o registro no banco
-                entity = self._dao.insert(entity)
+                entity = self._dao.insert(entity, dto.sql_read_only_fields)
 
                 # Inserindo os conjuntos (se necessário)
                 if self._dto_class.conjunto_type is not None:
@@ -633,13 +998,18 @@ class ServiceBase:
                         id, conjunto_field_value, self._dto_class.conjunto_type
                     )
             else:
+                # Suporte para evitar que o usuário possa alterar esse campo e que seja atualizado pelo sistema
+                if "atualizado_em" in dto.sql_read_only_fields:
+                    dto.sql_read_only_fields.remove("atualizado_em")
                 # Executando o update pelo DAO
                 entity = self._dao.update(
-                    entity_key_field,
-                    entity_id_value,
+                    entity.get_pk_field(),
+                    getattr(old_dto, dto.pk_field),
                     entity,
                     aditional_entity_filters,
                     partial_update,
+                    dto.sql_read_only_fields,
+                    upsert
                 )
 
             # Convertendo a entity para o DTO de resposta (se houver um)
@@ -755,14 +1125,18 @@ class ServiceBase:
             )
 
             # Getting service instance
-            # TODO Refatorar para suportar services customizados
-            detail_service = ServiceBase(
-                self._injector_factory,
-                detail_dao,
-                list_field.dto_type,
-                list_field.entity_type,
-                list_field.dto_post_response_type,
-            )
+            if list_field.service_name is not None:
+                detail_service = self._injector_factory.get_service_by_name(
+                    list_field.service_name
+                )
+            else:
+                detail_service = ServiceBase(
+                    self._injector_factory,
+                    detail_dao,
+                    list_field.dto_type,
+                    list_field.entity_type,
+                    list_field.dto_post_response_type,
+                )
 
             # Resolvendo a chave do relacionamento
             relation_key_field = entity.get_pk_field()
@@ -853,6 +1227,7 @@ class ServiceBase:
                     partial_update,
                     relation_field_map,
                     item["detail_pk"],
+                    aditional_filters=aditional_filters,
                 )
 
                 # Guardando o DTO na lista de retorno
@@ -869,6 +1244,21 @@ class ServiceBase:
     def delete(self, id: Any, additional_filters: Dict[str, Any] = None) -> DTOBase:
         self._delete(id, manage_transaction=True, additional_filters=additional_filters)
 
+    def delete_list(self,ids: list, additional_filters: Dict[str, Any] = None):
+
+        try:
+            self._dao.begin()
+
+            for _id in ids:
+                self._delete(_id, manage_transaction=True, additional_filters=additional_filters)
+
+        except:
+            self._dao.rollback()
+            raise
+        finally:
+            self._dao.commit()
+
+
     def entity_exists(
         self,
         entity: EntityBase,
@@ -900,8 +1290,7 @@ class ServiceBase:
         entity: EntityBase,
         entity_filters: Dict[str, List[Filter]],
         unique: Set[str],
-        entity_key_field: str,
-        entity_key_value: Any = None,
+        old_dto: DTOBase,
     ):
         # Tratando dos filtros recebidos (de partição), e adicionando os filtros da unique
         unique_filter = {}
@@ -916,8 +1305,8 @@ class ServiceBase:
         unique_entity_filters = self._create_entity_filters(unique_filter)
 
         # Removendo o campo chave, se estiver no filtro
-        if entity_key_field in unique_entity_filters:
-            del unique_entity_filters[entity_key_field]
+        if entity.get_pk_field() in unique_entity_filters:
+            del unique_entity_filters[entity.get_pk_field()]
 
         # Se não há mais campos na unique, não há o que validar
         if len(unique_entity_filters) <= 0:
@@ -927,9 +1316,17 @@ class ServiceBase:
         entity_filters = {**entity_filters, **unique_entity_filters}
 
         # Montando filtro de PK diferente (se necessário, isto é, se for update)
-        if entity_key_value is not None:
-            filters_pk = entity_filters.setdefault(entity_key_field, [])
-            filters_pk.append(Filter(FilterOperator.DIFFERENT, entity_key_value))
+        filters_pk = entity_filters.setdefault(entity.get_pk_field(), [])
+        filters_pk.append(
+            Filter(
+                FilterOperator.DIFFERENT,
+                (
+                    getattr(old_dto, dto.pk_field)
+                    if old_dto is not None
+                    else getattr(dto, dto.pk_field)
+                ),
+            )
+        )
 
         # Searching entity in DB
         try:
@@ -973,15 +1370,14 @@ class ServiceBase:
             if len(self._dto_class.list_fields_map) > 0:
                 self._delete_related_lists(id, additional_filters)
 
-            # Excluindo a entity principal
-            self._dao.delete(entity_filters)
-
             # Excluindo os conjuntos (se necessário)
             if self._dto_class.conjunto_type is not None:
                 self._dao.delete_relacionamento_conjunto(
                     id, self._dto_class.conjunto_type
                 )
 
+            # Excluindo a entity principal
+            self._dao.delete(entity_filters)
         except:
             if manage_transaction:
                 self._dao.rollback()
@@ -994,13 +1390,19 @@ class ServiceBase:
         # Handling each related list
         for _, list_field in self._dto_class.list_fields_map.items():
             # Getting service instance
-            # TODO Refatorar para suportar services customizados
-            service = ServiceBase(
-                self._injector_factory,
-                DAOBase(self._injector_factory.db_adapter(), list_field.entity_type),
-                list_field.dto_type,
-                list_field.entity_type,
-            )
+            if list_field.service_name is not None:
+                service = self._injector_factory.get_service_by_name(
+                    list_field.service_name
+                )
+            else:
+                service = ServiceBase(
+                    self._injector_factory,
+                    DAOBase(
+                        self._injector_factory.db_adapter(), list_field.entity_type
+                    ),
+                    list_field.dto_type,
+                    list_field.entity_type,
+                )
 
             # Making filter to relation
             filters = {
@@ -1143,10 +1545,11 @@ class ServiceBase:
                     # Recuperando a configuração do campo left join
                     left_join_field: DTOLeftJoinField = dto.left_join_fields_map[field]
 
-                    # Recuperando o valor da propriedade no DTO relacionado
-                    field_value = getattr(
-                        related_dto, left_join_field.related_dto_field
-                    )
+                    if related_dto is not None:
+                        # Recuperando o valor da propriedade no DTO relacionado
+                        field_value = getattr(
+                            related_dto, left_join_field.related_dto_field
+                        )
 
-                    # Gravando o valor no DTO de interesse
-                    setattr(dto, field, field_value)
+                        # Gravando o valor no DTO de interesse
+                        setattr(dto, field, field_value)
